Unit ACFronta;

//////////////////////////////////////////
(*
 Obecna implementace fornty. Fronta je
 vyuzita v algoritmu Aho-Corasickove, ale
 tuto implementaci lze s lehkou modifikaci
 vyuzit kdekoliv jinde.
*)
//////////////////////////////////////////

Interface

Type
    // prvek fronty
    TQueueItem = Integer;
    // struktura fronty
    PQueue = ^TQueue;
    TQueue = Record
             MaxSize : Integer;
             First : Integer;
             Last : Integer;
             Items : Array [0..1] Of TQueueItem;
             end;

// Inicializuje frontu
Function  QueueInit(AMaxSize:Integer):PQueue;
// Zrusi frontu
Procedure QueueDestroy(AQueue:PQueue);
// Odstrani prvni prvek z fronty
Function  QueueRemoveFirst(AQueue:PQueue):TQueueItem;
// Prida prvek na konec fronty
Procedure QueueAdd(Var AQueue:PQueue; AItem:TQueueItem);
// Zjisti, zda je fronta prazdna
Function  QueueEmpty(AQueue:PQueue):Boolean;

Implementation

Function QueueInit(AMaxSize:Integer):PQueue;
(*
 Inicializuje frontu.
 MaxSize - pocet polozek, pro ktere se
 ma alokovat pamet. Pokud by pretekl,
 dojde automaticky k realokaci.
 Vraci adresu nove vytvorene fronty.
*)
begin
GetMem(Result,AMaxSize*SizeOf(TQueueItem)+SizeOf(TQueue));
Result.MaxSize:=AMaxSize;
Result.First:=-1;
Result.Last:=-1;
end;

Procedure QueueDestroy(AQueue:PQueue);
(*
 Zrusi ridici strukturu fronty.
 AQueue - fronta, ktera se ma zrusit.
*)
begin
FreeMem(AQueue);
end;

Function  QueueRemoveFirst(AQueue:PQueue):TQueueItem;
(*
 Odstrani prvek z fronty.
 AQueue - fronta
 Vrati odstraneny prvek.
*)
begin
Result:=AQueue.Items[AQueue.First];
If AQueue.First <> AQueue.Last Then
   begin
   // Fronta jeste stale obsahuje prvky
   Inc(AQueue.First);
   If AQueue.First = AQueue.MaxSize Then
      AQueue.First:=0;
   end
Else begin
     // Odebran posledni prvek
     AQueue.First:=-1;
     AQueue.Last:=-1;
     end;
end;

Procedure QueueAdd(Var AQueue:PQueue; AItem:TQueueItem);
(*
 Prida prvek do fronty.
 AQueue - fronta.
 AItem - prvek, ktery se ma pridat.

 Pokud pri pridavani dojde k preteceni
 alokovane pameti pro frontu, dojde
 k realokaci, pricemz se alokuje prostor
 dvakrat vetsi.
*)
Var Q : PQueue;
    X : TQueueItem;
begin
Inc(AQueue.Last);
If AQueue.Last = AQueue.MaxSize Then
   AQueue.Last:=0;
If AQueue.Last = AQueue.First Then
   begin
   // Fronta jiz je plna
   // Musime ji realokovat
   Q:=QueueInit(AQueue.MaxSize*2);
   While Not QueueEmpty(AQueue) Do
         begin
         X:=QueueRemoveFirst(AQueue);
         QueueAdd(Q,X);
         end;
   QueueDestroy(AQueue);
   AQueue:=Q;
   end
Else begin
     AQueue.Items[AQueue.Last]:=AItem;
     If AQueue.First = -1 Then
        // Byl vlozen prvni prvek
        AQueue.First:=AQueue.Last;
     end;
end;

Function  QueueEmpty(AQueue:PQueue):Boolean;
(*
 Zjisti, zda je fronta prazdna.
 AQueue - dotazovana fronta.
 Vrati TRUE v pripadne prazdnosti,
 FALSE, je-li fronta neprazdna.
*)
begin
Result:= (AQueue.First = -1) And
         (AQueue.Last = -1);
end;

End.
