Unit ACHashTable;

//////////////////////////////////////////
(*
 Implementace hashovaci tabulky pouzita
 v algoritmu Aho-Corasickove pro doprednou
 funkci G.
*)
//////////////////////////////////////////

Interface

// Pocet slotu v tabulce
Const M =  65537;

Type
      // Polozka hashovaci tabulky
      PItem = ^TItem;
      TItem = Record
              // Odkaz na dalsi polozku
              Next     : PItem;
              // Vstupni stav
              State    : Integer;
              // Vstupni pismeno
              Zn       : Char;
              // Vystupni stav
              OutState : Integer;
              end;
      THashTable = Array [0..M-1] Of PItem;
      PHashTable = ^THashTable;

Function  HTInit:PHashTable;
Procedure HTDestroy(AHT:PHashTable);
Procedure HTAdd(AHT:PHashTable; AState:Integer; AZn:Char; AOutState:Integer);
Function  HTSearch(AHT:PHashTable; AQ:Integer; AZn:Char):Integer;

Implementation

Uses ACDopredna;

Function HTInit:PHashTable;
(*
 Inicializuje hashovaci tabulku.
 Vrati pointer na novou strukturu.
*)
begin
GetMem(Result,SizeOf(THashTable));
FillChar(Result^,SizeOf(THashTable),0);
end;

Procedure HTDestroy(AHT:PHashTable);
(*
 Uvolni pamet pouzivanou zadanou hashovaci
 tabulkou.
 AHT - tabulka, kterou chceme uvolnit z
 pameti.
*)
Var I : Integer;
    Item : PItem;
    Tmp  : PItem;
begin
For I:=0 To M-1 Do
    begin
    Item:=AHT[I];
    While Assigned(Item) Do
          begin
          Tmp:=Item;
          Item:=Item.Next;
          FreeMem(Tmp);
          end;
    end;
FreeMem(AHT);
end;

Function HTHash(AState:Integer; AZn:Char):Integer;
(*
 Hashovaci funkce.
 AState - pocatecni stav do funkce G.
 AZn - znak do funkce G.
 Vrati cilovy stav funkce G.
*)
begin
Result:= ((AState Shl 8) + Ord(AZn)) Mod M;
end;

Procedure HTAdd(AHT:PHashTable; AState:Integer; AZn:Char; AOutState:Integer);
(*
 Prida polozku do hashovaci tabulky.
 AHT - Hshovaci tabulka.
 AState - pocatecni stav.
 AZn - znak.
 AOutState - vystupni stav.
*)
Var H : Integer;
    Item : PItem;
begin
H:=HTHash(AState,AZn);
New(Item);
Item.Next:=AHT[H];
Item.State:=AState;
Item.Zn:=AZn;
Item.OutState:=AOutState;
AHT[H]:=Item;
end;

Function  HTSearch(AHT:PHashTable; AQ:Integer; AZn:Char):Integer;
(*
 Vraci informaci o tom, zda dana polozka
 existuje v tabulce.
 AHT - hashovaci tabulka.
 AQ - pocatecni stav.
 AZn - pocatecni stav.
 Vrati G_UNDEFINED, pokud se polozka v tabulce
 nenachazi, jinak vrati jeji cilovy stav.
*)
Var H : Integer;
    Item : PItem;
begin
Result:=G_UNDEFINED;
H:=HTHash(AQ,AZn);
Item:=AHT[H];
While Assigned(Item) Do
      begin
      If (Item.State = AQ) And
         (Item.Zn = AZn) Then
         begin
         Result:=Item.OutState;
         Break;
         end;
      Item:=Item.Next;
      end;
end;

End.
