Unit AhoCorasickAlg;

Interface

Uses
    Classes,
    ACFronta,
    TextSearcAlg;

Type
   // Definice typu reprezentujicich
   // zpetnou funkci F a vystupni funkci
   // Out
   TVystupniFunkce = Array Of Array Of Cardinal;
   TZpetnaFunkce = Array Of Integer;

   // Trida reprezentujici algoritmus
   // Aho-Corasickove
   TAhoCorasickAlgorithm = Class (TTextSearchAlgorithm)
                           Private
                            // Zpetna funkce F
                            FF : TZpetnaFunkce;
                            // Vystupni funkce OUT
                            // vraci index vzorku
                            FOut : TVystupniFunkce;
                            FO : TVystupniFunkce;
                            // Pocet stavu vyhledavaciho stroje
                            FPocetStavu : Integer;
                            // Najde vsechny vyskyty vzorku
                            Procedure Algorithm1;
                            // Definuje G a castecne Out
                            Procedure Algorithm2;
                            // Definuje F a dodefinuje zbytek Out
                            Procedure Algorithm3;
                           Public
                            // Provede vlastni vyhledavani
                            Procedure DoSearch; Override;
                            // Vytvori instanci algoritmu
                            // Aho-Corasickove.
                            Constructor Create(AText:String; APatterns:TStringList; AAlgName:String); Override;
                           end;


Implementation

Uses ACDopredna;

Procedure TAhoCorasickAlgorithm.Algorithm1;
(*
 Prohleda zadany text a najde vsechny
 usporadane dvojice (i, v), kde i
 je pozice prvniho pismene vyskytu vzorku
 v v textu T.
*)
Var J : Integer;
    I : Cardinal;
    S : Integer;
begin
S:=0;
For I:=1 To Length(FText) Do
    begin
    While (G(S,FText[I]) = G_UNDEFINED) Do
           S:=FF[S];
    S:=G(S,FText[I]);
    For J:=0 To High(FOut[S]) Do
        OutputAdd(I-Length(FPatterns[FOut[S][J]])+1,FOut[S][J]);
    end;
DestroyG;
end;

Procedure TAhoCorasickAlgorithm.Algorithm2;
(*
 Definuje doprednou funkci pro vyhledavaci
 automat a take definuje nektere hodnoty
 funkce vystupni.
*)

 Procedure Enter(APatternIndex:Integer);
 (*
  Definuje doprednou funkci pro dany vzorek.
  APatternIndex - index tohoto vzorku v
  seznamu vzorku.
 *)
 Var
     // Aktualni stav
     S : Integer;
     // Pro prochazeni obsahu vzorku
     J : Integer;
     // Delka retezce vzorku
     N : Integer;
     // Retezec vzorku
     P : String;
 begin
 S:=0;
 J:=1;
 P:=FPatterns[APatternIndex];
 // Dokud je dopredna funkce definovana
 // (uz jsme pridavali nejaky vzorek se
 // shodnou predponou), pokracujeme
 // po ni
 N:=Length(P);
 While (J <= N) And (G(S,P[J]) <> G_UNDEFINED) Do
       begin
       S:=G(S,P[J]);
       Inc(J);
       end;
 // Dopredna funkce jiz zde definovana neni.
 // Dodefinujeme ji, musime take pridat nove
 // stavy
 While J <= N Do
       begin
       DefinujG(S,P[J],FPocetStavu);
       S:=FPocetStavu;
       Inc(J);
       Inc(FPocetStavu);
       end;
 // Pridame hodnotu vystupni funkce pro
 // stav reprezentujici nas vzorek
 SetLength(FO,FPocetStavu);
 SetLength(FO[S],Length(FO[S])+1);
 FO[S][High(FO[S])]:=APatternIndex;
 end;

Var
    // Pro prochazeni seznamu vzorku
    K : Integer;
    // Pro prochazeni mnoziny abecedy
    I : Integer;
begin
InitG;
FPocetStavu:=1;
// Vlozime vsechny vzorky do vyhledavaciho
// stroje - definujeme pro ne doprednou
// funkci G
For K:=0 To FPatterns.Count-1 Do
    Enter(K);
// Dodefinujeme doprednou funkci pro vychozi
// stav pro vsechna pismena, pokud jiz neni
// definovana
For I:=0 To MAX_ABECEDA-1 Do
    begin
    // Diky nasi implementaci se musime podivat,
    // zda dany znak v nasi abecede vubec je
    If (FAbeceda[I] <> NENI_V_ABECEDE) And
       (G(0,Chr(I)) = G_UNDEFINED) Then
       // Pokud ze stavu 0 neni pro dany znak definovan
       // prechodovy stav, dodefinujeme jej
       DefinujG(0,Chr(I),0);
    end;
end;

Procedure TAhoCorasickAlgorithm.Algorithm3;
(*
 Definuje zpetnou funkci a dodefinuje zbytek
 funkce vystupni. Po dokonceni tohoto kroku
 je vyhledavaci stroj pripraven splnit svuj
 ukol.
*)
Var
    // Prochazeni abecedy
    I : Integer;
    // Aktualni stav
    S : Integer;
    // Pomocny stav
    R : Integer;
    // Pomocny stav
    T : Integer;
    // Fronta stavu
    Queue : PQueue;
    // Pruchod outem
    J : Integer;
    K : Integer;
begin
Queue:=QueueInit(FPocetStavu);
SetLength(FF,FPocetStavu);
SetLength(FOut,FPocetStavu);
SetLength(FOut[0],0);
FF[0]:=0;
For I:=0 To MAX_ABECEDA-1 Do
    begin
    If FAbeceda[I] <> NENI_V_ABECEDE Then
       begin
       S:=G(0,Chr(I));
       If S > 0 Then
          begin
          FF[S]:=0;
          FOut[S]:=Copy(FO[S],0,Length(FO[S]));
          QueueAdd(Queue,S);
          end;
       end;
    end;
While Not QueueEmpty(Queue) Do
      begin
      R:=QueueRemoveFirst(Queue);
      For I:=0 To MAX_ABECEDA-1 Do
          begin
          If FAbeceda[I] <> NENI_V_ABECEDE Then
             begin
             If G(R,Chr(I)) <> G_UNDEFINED Then
                begin
                // S = naslednik R
                S:=G(R,Chr(I));
                // T = predchudce R
                T:=FF[R];
                // Dokud nejde postupovat opet dopredu,
                // vracime se s T dozadu
                While G(T,Chr(I)) = G_UNDEFINED Do
                      T:=FF[T];
                // Jiz mame bod, odkud je mozne pokracovat z S,
                // definujeme tam F(s)
                FF[S]:=G(T,Chr(I));
                // Definovat out(s) = o(s) sjednoceno out(f(s)))
                FOut[S]:=Copy(FO[S],0,Length(FO[S]));
                For J:=0 To High(FOut[FF[S]]) Do
                    begin
                    SetLength(FOut[S],Length(FOut[S])+1);
                    FOut[S][High(FOut[S])]:=FOut[FF[S]][J];
                    end;
                // Pridej vrchol do fronty
                QueueAdd(Queue,S);
                end;
             end;
          end;
      end;
QueueDestroy(Queue);
end;

Procedure TAhoCorasickAlgorithm.DoSearch;
(*
 Prohleda zadany text algoritmem Aho-Corasickove.
 Nejprve provede veskerou inicializaci (priprava
 vyhledavaciho stroje, definice zpetny a vystupni
 funkce) a pak najde vsechny dvojice (i, v).
*)
begin
// Priprava vyhledavaciho stroje, tzn.
// definice dopredne funkce a castecne
// i funkce vystupni
Algorithm2;
// Definice zpetne funkce a dodefinovani
// funkce vystupni
Algorithm3;
// Prohledani textu a nalezeni vsech usporadanych
// dvojic (i, v)
Algorithm1;
// TODO: Cleanup
end;

Constructor TAhoCorasickAlgorithm.Create(AText:String; APatterns:TStringList; AAlgName:String);
(*
 Vytvori instanci algoritmu Aho-Corasickove.

 AText - text, ve kterem se bude vyhledavat.
 APatterns - co se bude vyhledavat (vzorky)
 AAlgName - nazev algoritmu (pro ucely tisku)
 *)
begin
Inherited Create(AText,APatterns,AAlgName);
// Inicializujeme hodnoty
SetLength(FF,0);
SetLength(FOut,0);
FPocetStavu:=0;
end;

End.
