Program TestApp;

//////////////////////////////////////////
(*
 Toto je jednoducha aplikace, pro testovani
 ruznych algoritmu na vyhledavani v textu,
 ktere resi ulohu zjisteni pozice vsech
 vyskytu zvorku v textu.

 Jedna se o konzolovou aplikaci.
 Syntaxe prikazove radky:

 TestApp <soubor_textu> <soubor_vzorku>

 <soubor_textu> je retezec obsahujici nazev
 souboru s textem, ktery se bude prohledavat.

 <soubor_vzorku> je nazev textoveho souboru, jenz
 na kazdem radku musi obsahovat prave jeden
 vzorek.
*)
//////////////////////////////////////////

// Prikaz pro prekladac, ze se jedna
// o konzolovou aplikaci
{$APPTYPE CONSOLE}

uses
  Classes,
  Windows,
  TextSearcAlg in 'TextSearcAlg.pas',
  NaiveAlg in 'NaiveAlg.pas',
  RabinKarpAlg in 'RabinKarpAlg.pas',
  AhoCorasickAlg in 'AhoCorasickAlg.pas',
  ACZpetna in 'ACZpetna.pas',
  ACDopredna in 'ACDopredna.pas',
  ACFronta in 'ACFronta.pas',
  ACHashTable in 'ACHashTable.pas';

Var
  // Naivni algoritmus
  NA : TNaiveAlg;
  RK : TRabinKarpAlgorithm;
  AC : TAhoCorasickAlgorithm;
  // Vzorky
  P  : TStringList;
  // Prohledavany text
  T  : String;

Procedure VypisVysledky(AAlg:TTextSearchAlgorithm);
(*
 Vypise vysledky prohledavani v textu.
 AAlg - prave dokonceny algoritmus, jehoz vysledky se maji zobrazit
*)
Var F : TextFile;
    I : Integer;
begin
Writeln('Vysledky algoritmu "',AAlg.AlgName,'"');
Writeln('Shod:  ',Length(AAlg.Outputs));
AssignFile(F,AAlg.AlgName+'.txt');
Rewrite(F);
Writeln(F,'INDEX    RETEZEC');
For I:=0 To High(AAlg.Outputs) Do
    Writeln(F,AAlg.Outputs[I].Index:5,'    ',P[AAlg.Outputs[I].PatternIndex]);
CloseFile(F);
end;


Var
    // Cas pred zacatkem algoritmu
    T1 : Int64;
    // Cas po vyhledavaci fazi algoritmu
    T2 : Int64;
    // Cas po vypsani vystupu algoritmu
    T3 : Int64;
Begin
// Potrebujeme dva vstupni parametry
If ParamCount = 2 Then
   begin
   // Nacteme text, ktery se bude prohledavat
   // (je to mozna osklive, ale kratke :-)).
   Writeln('Soubor textu.....',ParamStr(1));
   With TStringList.Create Do
        begin
        LoadFromFile(ParamStr(1));
        T:=Text;
        Free;
        end;
   // Nacteme vzorky
   Writeln('Soubor vzorku....',ParamStr(2));
   P:=TStringList.Create;
   P.LoadFromFile(ParamStr(2));

   // Zmerime dobu trvani naivniho algoritmu
   RK:=TRabinKarpAlgorithm.Create(T,P,'Rabin-Karp algorithm');
   T1:=GetTickCount;
   RK.DoSearch;
   T2:=GetTickCount;
   VypisVysledky(RK);
   T3:=GetTickCount;
   Writeln('Vyhledavaci faze: ',(T2-T1),' ms');
   Writeln('Vypis vysledku:   ',(T3-T2),' ms');
   Writeln('Celkovy cas:      ',(T3-T1),' ms');
   Writeln('----------------');
   RK.Free;

   // Zmerime dobu trvani naivniho algoritmu
   AC:=TAhoCorasickAlgorithm.Create(T,P,'Aho-Corasick algorithm');
   T1:=GetTickCount;
   AC.DoSearch;
   T2:=GetTickCount;
   VypisVysledky(AC);
   T3:=GetTickCount;
   Writeln('Vyhledavaci faze: ',(T2-T1),' ms');
   Writeln('Vypis vysledku:   ',(T3-T2),' ms');
   Writeln('Celkovy cas:      ',(T3-T1),' ms');
   Writeln('----------------');
   AC.Free;

   // Zmerime dobu trvani naivniho algoritmu
   NA:=TNaiveAlg.Create(T,P,'Naive algorithm');
   T1:=GetTickCount;
   NA.DoSearch;
   T2:=GetTickCount;
   VypisVysledky(NA);
   T3:=GetTickCount;
   Writeln('Vyhledavaci faze: ',(T2-T1),' ms');
   Writeln('Vypis vysledku:   ',(T3-T2),' ms');
   Writeln('Celkovy cas:      ',(T3-T1),' ms');
   NA.Free;

   // Uklid
   P.Free;
   end
Else begin
     // Spatny pocet parametru
     Writeln('TestApp <soubor_textu> <soubor_vzorku>');
     end;
End.
